@@ -1,4 +1,5 @@
 import argparse
+import calendar
 import concurrent.futures
 import datetime as dt
 import gzip
@@ -7,8 +8,10 @@ import shutil
 import sys
 import time
 
+import dataframe_image as dfi
 import osmium
 import pandas as pd
+import pytz
 import requests
 from osmium.replication.server import ReplicationServer
 
@@ -17,6 +20,61 @@ from osmsg.utils import verify_me_osm
 users_temp = {}
 users = {}
 
+from datetime import datetime, timedelta
+
+
+def strip_utc(date_str, timezone):
+    tz = dt.timezone.utc
+    if timezone == "Nepal":
+        # Set the timezone to Nepal
+        tz = pytz.timezone("Asia/Kathmandu")
+
+    return dt.datetime.strptime(date_str, "%Y-%m-%d").replace(tzinfo=tz)
+
+
+def get_prev_year_dates(timezone):
+    # Get today's date
+    today = datetime.now()
+
+    # Get the start and end date of the previous year
+    prev_year_start = datetime(today.year - 1, 1, 1)
+    prev_year_end = datetime(today.year - 1, 12, 31)
+
+    return strip_utc(prev_year_start.strftime("%Y-%m-%d"), timezone), strip_utc(
+        prev_year_end.strftime("%Y-%m-%d"), timezone
+    )
+
+
+def previous_month(timezone):
+    today = datetime.now()
+    month = today.month
+    year = today.year
+    if today.month == 1:
+        month = 13
+        year -= 1
+    prev_month_start = datetime(year, month - 1, 1)
+    prev_month_end = datetime(year, month - 1, calendar.monthrange(year, month - 1)[1])
+    return strip_utc(prev_month_start.strftime("%Y-%m-%d"), timezone), strip_utc(
+        prev_month_end.strftime("%Y-%m-%d"), timezone
+    )
+
+
+def previous_day(timezone):
+    today = datetime.today()
+    previous_day = today - timedelta(days=1)
+    return strip_utc(previous_day.strftime("%Y-%m-%d"), timezone), strip_utc(
+        today.strftime("%Y-%m-%d"), timezone
+    )
+
+
+def previous_week(timezone):
+    today = datetime.today()
+    start_date = today - timedelta(days=today.weekday() + 7)
+    end_date = start_date + timedelta(days=6)
+    return strip_utc(start_date.strftime("%Y-%m-%d"), timezone), strip_utc(
+        end_date.strftime("%Y-%m-%d"), timezone
+    )
+
 
 def calculate_stats(user, uname, changeset, version, tags, osm_type):
     tags_to_collect = list(additional_tags) if additional_tags else None
@@ -174,7 +232,7 @@ def get_download_urls_changefiles(start_date, end_date, base_url):
             )
         download_urls.append(seq_url)
         seq += 1
-    return download_urls, last_ts
+    return download_urls, server_ts
 
 
 def auth(username, password):
@@ -187,15 +245,20 @@ def auth(username, password):
 
 def main():
     parser = argparse.ArgumentParser()
-    parser.add_argument(
-        "--start_date", required=True, help="Start date in the format YYYY-MM-DD"
-    )
+    parser.add_argument("--start_date", help="Start date in the format YYYY-MM-DD")
     parser.add_argument("--end_date", help="End date in the format YYYY-MM-DD")
     parser.add_argument("--username", required=True, help="Your OSM Username")
     parser.add_argument("--password", required=True, help="Your OSM Password")
+    parser.add_argument(
+        "--timezone",
+        default="UTC",
+        choices=["Nepal", "UTC"],
+        help="Your Timezone : Currently Supported Nepal, Default : UTC",
+    )
+
     parser.add_argument(
         "--name",
-        default="output_stats",
+        default="stats",
         help="Output stat file name",
     )
     parser.add_argument(
@@ -205,22 +268,45 @@ def main():
         help="Additional stats to collect : List of tags key",
     )
 
+    parser.add_argument(
+        "--rows",
+        type=int,
+        help="No fo top rows to extract , to extract top 100 , pass 100",
+    )
+
     parser.add_argument(
         "--url", required=True, help="Your public Geofabrik Download URL "
     )
+    parser.add_argument("--extract_last_week", action="store_true", default=False)
+    parser.add_argument("--extract_last_day", action="store_true", default=False)
+    parser.add_argument("--extract_last_month", action="store_true", default=False)
+    parser.add_argument("--extract_last_year", action="store_true", default=False)
+
     parser.add_argument(
         "--format",
-        choices=["csv", "json", "excel"],
+        choices=["csv", "json", "excel", "image"],
         default="json",
         help="Stats output format",
     )
 
     args = parser.parse_args()
-    start_date = dt.datetime.strptime(args.start_date, "%Y-%m-%d").replace(
-        tzinfo=dt.timezone.utc
-    )
+    if args.start_date:
+        start_date = dt.datetime.strptime(args.start_date, "%Y-%m-%d").replace(
+            tzinfo=dt.timezone.utc
+        )
+
+    if not args.start_date:
+        if (
+            args.extract_last_week
+            or args.extract_last_day
+            or args.extract_last_month
+            or args.extract_last_year
+        ):
+            pass
+        else:
+            print("Supply start_date")
+            sys.exit()
 
-    end_date = None
     if args.end_date:
         end_date = dt.datetime.strptime(args.end_date, "%Y-%m-%d").replace(
             tzinfo=dt.timezone.utc
@@ -234,10 +320,23 @@ def main():
     if "geofabrik" in args.url:
         cookies = auth(args.username, args.password)
     print("Script Started")
+
+    if args.extract_last_year:
+        start_date, end_date = get_prev_year_dates(args.timezone)
+
+    if args.extract_last_month:
+        start_date, end_date = previous_month(args.timezone)
+
+    if args.extract_last_day:
+        start_date, end_date = previous_day(args.timezone)
+    if args.extract_last_week:
+        start_date, end_date = previous_week(args.timezone)
     print("Generating Download Urls")
     download_urls, server_ts = get_download_urls_changefiles(
         start_date, end_date, args.url
     )
+    if server_ts < end_date:
+        end_date = server_ts
     print("Download urls Generated")
 
     print("Starting Thread Processing")
@@ -245,32 +344,41 @@ def main():
     with concurrent.futures.ThreadPoolExecutor() as executor:
         # Use `map` to apply the `download_image` function to each element in the `urls` list
         executor.map(process_changefiles, download_urls)
+    if len(users) > 1:
 
-    df = pd.json_normalize(list(users.values()))
-    df = df.assign(
-        changes=df["nodes.create"]
-        + df["nodes.modify"]
-        + df["nodes.delete"]
-        + df["ways.create"]
-        + df["ways.modify"]
-        + df["ways.delete"]
-        + df["relations.create"]
-        + df["relations.modify"]
-        + df["relations.delete"]
-    )
-    df.insert(3, "map_changes", df["changes"], True)
-    df = df.drop(columns=["changes"])
-    df = df.sort_values("map_changes", ascending=False)
-    df.insert(0, "rank", range(1, len(df) + 1), True)
-    print(df)
-    if args.format == "json":
-        # with open(f"{out_file_name}.json") as file:
-        #     file.write(json.dumps(users))
-        df.to_json(f"{args.name}.json", orient="records")
-    if args.format == "csv":
-        df.to_csv(f"{args.name}.csv", index=False)
-    if args.format == "excel":
-        df.to_excel(f"{args.name}.xlsx", index=False)
+        df = pd.json_normalize(list(users.values()))
+        df = df.assign(
+            changes=df["nodes.create"]
+            + df["nodes.modify"]
+            + df["nodes.delete"]
+            + df["ways.create"]
+            + df["ways.modify"]
+            + df["ways.delete"]
+            + df["relations.create"]
+            + df["relations.modify"]
+            + df["relations.delete"]
+        )
+        df.insert(3, "map_changes", df["changes"], True)
+        df = df.drop(columns=["changes"])
+        df = df.sort_values("map_changes", ascending=False)
+        df.insert(0, "rank", range(1, len(df) + 1), True)
+        if args.rows:
+            df = df.head(args.rows)
+        print(df)
+        if args.format == "image":
+            # Convert the DataFrame to an image
+            dfi.export(df, f"{args.name}_{start_date}_{end_date}.png")
+
+        if args.format == "json":
+            # with open(f"{out_file_name}.json") as file:
+            #     file.write(json.dumps(users))
+            df.to_json(f"{args.name}_{start_date}_{end_date}.json", orient="records")
+        if args.format == "csv":
+            df.to_csv(f"{args.name}_{start_date}_{end_date}.csv", index=False)
+        if args.format == "excel":
+            df.to_excel(f"{args.name}_{start_date}_{end_date}.xlsx", index=False)
+    else:
+        sys.exit()
 
     end_time = time.time()
     elapsed_time = end_time - start_time
